@@ -1,12 +1,15 @@
+## mylevels() returns levels if given a factor, otherwise 0.
+mylevels <- function(x) if (is.factor(x)) levels(x) else 0
+
 "modified.randomForest.default" <-
     function(x, y=NULL,  xtest=NULL, ytest=NULL, ntree=500,
-             mtry=if (!is.null(y) && !is.factor(y))
+             mtry=if (!is.null(y) && !is.factor(y)) 
              max(floor(ncol(x)/3), 1) else floor(sqrt(ncol(x))),
              replace=TRUE, classwt=NULL, cutoff, strata,
              sampsize = if (replace) nrow(x) else ceiling(.632*nrow(x)),
-             nodesize = if (!is.null(y) && !is.factor(y)) 5 else 1,
+             nodesize = if (!is.null(y) && !is.factor(y)) 5 else 1, 
              importance=FALSE, localImp=FALSE, nPerm=1,
-             proximity=FALSE, oob.prox=proximity,
+             proximity, oob.prox=proximity,
              norm.votes=TRUE, do.trace=FALSE,
              keep.forest=!is.null(y) && is.null(xtest), corr.bias=FALSE,
              keep.inbag=FALSE, ...) {
@@ -15,21 +18,21 @@
     if (!classRF && length(unique(y)) <= 5) {
         warning("The response has five or fewer unique values.  Are you sure you want to do regression?")
     }
-    if (classRF && !addclass && length(unique(y)) < 2)
+    if (classRF && !addclass && length(unique(y)) < 2) 
         stop("Need at least two classes to do classification.")
     n <- nrow(x)
     p <- ncol(x)
     if (n == 0) stop("data (x) has 0 rows")
     x.row.names <- rownames(x)
     x.col.names <- if (is.null(colnames(x))) 1:ncol(x) else colnames(x)
-
+    
     ## overcome R's lazy evaluation:
     keep.forest <- keep.forest
-
+    
     testdat <- !is.null(xtest)
     if (testdat) {
         if (ncol(x) != ncol(xtest))
-            stop("x and xtest must have same number of columns")
+            stop("x and xtest must have same number of columns") 
         ntest <- nrow(xtest)
         xts.row.names <- rownames(xtest)
     }
@@ -54,26 +57,34 @@
     if (!is.null(ytest) && any(is.na(ytest))) stop("NA not permitted in ytest")
 
     if (is.data.frame(x)) {
-        ncat <- sapply(x, function(x) if(is.factor(x) && !is.ordered(x))
-                       length(levels(x)) else 1)
+        xlevels <- lapply(x, mylevels)
+        ncat <- sapply(xlevels, length)
+        ## Treat ordered factors as numerics.
+        ncat <- ifelse(sapply(x, is.ordered), 1, ncat)
         x <- data.matrix(x)
         if(testdat) {
             if(!is.data.frame(xtest))
                 stop("xtest must be data frame if x is")
-            ncatts <- sapply(xtest, function(x) if(is.factor(x) &&
-                                                   !is.ordered(x))
-                             length(levels(x)) else 1)
-            if(!all(ncat == ncatts))
-                stop("columns of x and xtest must be the same type")
+            xfactor <- which(sapply(xtest, is.factor))
+            if (length(xfactor) > 0) {
+                for (i in xfactor) {
+                    if (any(! levels(xtest[[i]]) %in% xlevels[[i]]))
+                        stop("New factor levels in xtest not present in x")
+                    xtest[[i]] <-
+                        factor(xlevels[[i]][match(xtest[[i]], xlevels[[i]])],
+                               levels=xlevels[[i]])
+                }
+            }
             xtest <- data.matrix(xtest)
         }
     } else {
         ncat <- rep(1, p)
+        xlevels <- as.list(rep(0, p))
     }
     maxcat <- max(ncat)
     if (maxcat > 32)
         stop("Can not handle categorical predictors with more than 32 categories.")
-
+    
     if (classRF) {
         nclass <- length(levels(y))
         ## Check for empty classes:
@@ -113,23 +124,20 @@
             classwt <- rep(1, nclass)
             ipi <- 0
         }
-    } else addclass <- FALSE
-##nlc:
-##we don't want the proximity unless we ask for it...so the following line is
-##commented out:
-#    if (addclass) proximity <- TRUE
+    } else addclass <- FALSE  
+
+    if (missing(proximity)) proximity <- addclass
     if (proximity) {
         prox <- matrix(0.0, n, n)
         proxts <- if (testdat) matrix(0, ntest, ntest + n) else double(1)
     } else {
         prox <- proxts <- double(1)
     }
-
     if (localImp) {
         importance <- TRUE
         impmat <- matrix(0, p, n)
     } else impmat <- double(1)
-
+    
     if (importance) {
         if (nPerm < 1) nPerm <- as.integer(1) else nPerm <- as.integer(nPerm)
         if (classRF) {
@@ -144,7 +152,7 @@
         impout <- double(p)
         impSD <- double(1)
     }
-
+    
     nsample <- if (addclass) 2 * n else n
     Stratify <- length(sampsize) > 1
     if ((!Stratify) && sampsize > nrow(x)) stop("sampsize too large")
@@ -172,7 +180,7 @@
         ## For regression trees, need to do this to get maximal trees.
         nrnodes <- 2 * trunc(sampsize/max(1, nodesize - 4)) + 1
     }
-
+    
     ## Compiled code expects variables in rows and observations in columns.
     x <- t(x)
     storage.mode(x) <- "double"
@@ -199,7 +207,7 @@
                     xdim = as.integer(c(p, n)),
                     y = as.integer(y),
                     nclass = as.integer(nclass),
-                    ncat = as.integer(ncat),
+                    ncat = as.integer(ncat), 
                     maxcat = as.integer(maxcat),
                     sampsize = as.integer(sampsize),
                     strata = if (Stratify) as.integer(strata) else integer(1),
@@ -242,10 +250,10 @@
                     labelts = as.integer(labelts),
                     proxts = proxts,
                     errts = error.test,
-                    inbag = if (keep.inbag)
+                    inbag = if (keep.inbag) 
                     matrix(integer(n * ntree), n) else integer(n),
                     DUP=FALSE,
-                    PACKAGE="randomForest")[-1]
+                    PACKAGE="randomForest")[-1] 
         if (keep.forest) {
             ## deal with the random forest outputs
             max.nodes <- max(rfout$ndbigtree)
@@ -263,7 +271,7 @@
         }
         out.votes <- t(matrix(rfout$counttr, nclass, nsample))[1:n, ]
         oob.times <- rowSums(out.votes)
-        if(norm.votes)
+        if(norm.votes) 
             out.votes <- t(apply(out.votes, 1, function(x) x/sum(x)))
         dimnames(out.votes) <- list(x.row.names, levels(y))
         if(testdat) {
@@ -294,7 +302,7 @@
                     votes = out.votes,
                     oob.times = oob.times,
                     classes = levels(y),
-                    importance = if (importance)
+                    importance = if (importance) 
                     matrix(rfout$impout, p, nclass+2,
                            dimnames = list(x.col.names,
                            c(levels(y), "MeanDecreaseAccuracy",
@@ -314,7 +322,7 @@
                     ntree = ntree,
                     mtry = mtry,
                     forest = if (!keep.forest) NULL else {
-                        list(ndbigtree = rfout$ndbigtree,
+                        list(ndbigtree = rfout$ndbigtree, 
                              nodestatus = matrix(rfout$nodestatus,
                              nc = ntree)[1:max.nodes,, drop=FALSE],
                              bestvar = matrix(rfout$bestvar, nc = ntree)[1:max.nodes,, drop=FALSE],
@@ -323,8 +331,10 @@
                              nc = ntree)[1:max.nodes,, drop=FALSE],
                              xbestsplit = matrix(rfout$xbestsplit,
                              nc = ntree)[1:max.nodes,, drop=FALSE],
-                             pid = rfout$classwt, cutoff = cutoff, ncat = ncat, maxcat = maxcat,
-                             nrnodes = max.nodes, ntree = ntree, nclass = nclass)
+                             pid = rfout$classwt, cutoff=cutoff, ncat=ncat,
+                             maxcat = maxcat, 
+                             nrnodes = max.nodes, ntree = ntree,
+                             nclass = nclass, xlevels=xlevels)
                     },
                     y = if (addclass) NULL else y,
                     test = if(!testdat) NULL else list(
@@ -380,7 +390,7 @@
                     msets = double(if (labelts) ntree else 1),
                     coef = double(2),
                     oob.times = integer(n),
-                    inbag = if (keep.inbag)
+                    inbag = if (keep.inbag) 
                     matrix(integer(n * ntree), n) else integer(1),
                     DUP=FALSE,
                     PACKAGE="randomForest")[c(16:28, 36:41)]
@@ -414,7 +424,7 @@
                         matrix(rfout$impout, ncol=1,
                                dimnames=list(x.col.names, "IncNodePurity")),
                     importanceSD=if (importance) rfout$impSD else NULL,
-                    localImportance = if (importance)
+                    localImportance = if (localImp)
                     matrix(rfout$impmat, p, n, dimnames=list(x.col.names,
                                                x.row.names)) else NULL,
                     proximity = if (proximity) matrix(rfout$prox, n, n,
@@ -426,7 +436,7 @@
                               "rightDaughter", "nodepred", "bestvar",
                               "xbestsplit")],
                       list(ncat = ncat), list(nrnodes=max.nodes),
-                      list(ntree=ntree)) else NULL,
+                      list(ntree=ntree), list(xlevels=xlevels)) else NULL,
                     coefs = if (corr.bias) rfout$coef else NULL,
                     y = y,
                     test = if(testdat) {
@@ -435,7 +445,7 @@
                              mse = if(labelts) rfout$msets else NULL,
                              rsq = if(labelts) 1 - rfout$msets /
                                         (var(ytest) * (n-1) / n) else NULL,
-                             proximity = if (proximity)
+                             proximity = if (proximity) 
                              matrix(rfout$proxts / ntree, nrow = ntest,
                                     dimnames = list(xts.row.names,
                                     c(xts.row.names,
